// ----------------------------------------------------------------------------
// Copyright (C) 2018
//              David Freese, W1HKJ
//
// This file is part of pskmeter.
//
// pskmeter is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// pskmeter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------

#include <iostream>

#include "dataloop.h"

#include <config.h>

#include "threads.h"
#include "support.h"
#include "dataloop.h"
#include "util.h"

static pthread_t dataloop_thread;
//static pthread_mutex_t dataloop_mutex = PTHREAD_MUTEX_INITIALIZER;

static bool dataloop_exit = false;

void readdata()
{
	double vals[NUMPTS+1];
	int ival[NUMPTS+1];
// change for debugging
#if 1
	pskserial->WriteByte('s');
	MilliSleep(200);
	std::string sbuffer;
	char buffer[2048];
	memset(buffer, 0, 2048);
	pskserial->ReadBuffer (buffer, sizeof(buffer));
	sbuffer = buffer;
	size_t p = 0;
	for (int i = 0; i <= NUMPTS; i++) {
		ival[i] = 0;
		if (p != std::string::npos) {
			if (sscanf(&sbuffer[p], "%d", &ival[i])) {
				p = sbuffer.find(",", p+1);
				if (p != std::string::npos) p += 2;
			}
		}
		vals[i] = 5.0 * ival[i] / A2DSIZE;
	}

	pskdata->data(vals); 
#else  // debugging emulation
// standard waveform
#	if 1
	double p = exp10(sldrSimulate->value()/20.0);
	double sig = 0;
	int shift = (int)(64.0 * random() / RAND_MAX);

	for (int i = 0; i < NUMPTS; i++) {
		sig = sin(TWOPI*(i + shift)/NUMPTS);
		for (int q = 3; q <= 7; q += 2)
			sig += (9*p/(q*q))*sin(TWOPI*q*(i + shift)/NUMPTS);
		sig = fabs((int)(sig * A2DSIZE));
		vals[i] = 5.0 * sig / A2DSIZE;
	}
// captured waveform
#	else
	int td[] = {
		630,560,477,405,319,234,163,82,
		38,92,166,248,321,407,494,569,
		629,670,726,748,773,792,798,807,
		806,802,794,775,762,728,695,652,
		594,526,440,356,285,197,123,59,
		62,136,199,284,364,449,528,595,
		655,702,732,763,781,798,802,804,
		814,803,792,778,745,725,681,633 };

	for (int i = 0; i < NUMPTS; i++) {
		vals[i] = 5.0 * td[i] / 1024.0;
	}
#	endif
	MilliSleep(200);
	pskdata->data(vals);
#endif
}

void *data_loop(void *args)
{
	while(1) {
		MilliSleep(10);
		if (dataloop_exit) break;
		if (connected && bEnabled && pskdata->ready())
			readdata();
	}
// exit the dataloop thread
	return NULL;
}

//======================================================================
//
//======================================================================
bool start_dataloop()
{
	dataloop_exit = false;

	if (pthread_create(&dataloop_thread, NULL, data_loop, NULL) < 0) {
		std::cout << "create pthread failed" << std::endl;
		return false;
	}
	return true;
}

//======================================================================
//
//======================================================================
void close_dataloop()
{
	dataloop_exit = true;
	pthread_join(dataloop_thread, NULL);
}
