//======================================================================
// Copyright (C) 2018
//              David Freese, W1HKJ
//
// This file is part of pskscope
//
// fldigi is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// fldigi is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//======================================================================

#include <iostream>
#include <fstream>

#include <FL/Fl_Preferences.H>

#include <config.h>

#include "status.h"

#define status_FILENAME PACKAGE_TARNAME

std::string HomeDir = "./";

status::status() 
{
	serial_port = "NONE";

	mainX = 50;
	mainY = 50;

	serial_port = "NONE";
	minval = 8;
	overdrive1 = -24.0;
	overdrive2 = -18.0;
	alpha = 0.85;
	r1qro = 39.0;
	r1qrp = 6.8;
	targetpwr = 15.0;
	qrp = 0; // false

}

void status::saveLastState()
{
	int mX = pskscope_dialog->x();
	int mY = pskscope_dialog->y();
	if (mX >= 0 && mX >= 0) {
		mainX = mX;
		mainY = mY;
	}

#if FLDIGI_FLTK_API_MINOR < 4
	Fl_Preferences spref(HomeDir.c_str(), "w1hkj.com",  PACKAGE_NAME);
#else
	Fl_Preferences spref(std::string(HomeDir).append("flscope.prefs").c_str(), "w1hkj", PACKAGE_TARNAME, (Fl_Preferences::Root)0);
#endif

	spref.set("version", PACKAGE_VERSION);

	spref.set("main_x", mainX);
	spref.set("main_y", mainY);

	spref.set("serial_port", serial_port.c_str());
	spref.set("minval", minval, minval);

	spref.set("overdrive1", overdrive1);
	spref.set("overdrive2", overdrive2);
	spref.set("alpha", alpha);
	spref.set("r1qro", r1qro);
	spref.set("r1qrp", r1qrp);
	spref.set("targetpwr", targetpwr);
	spref.set("qrp", qrp);
}

void status::loadLastState()
{
#if FLDIGI_FLTK_API_MINOR < 4
	Fl_Preferences spref(HomeDir.c_str(), "w1hkj.com",  PACKAGE_NAME);
#else
	Fl_Preferences spref(std::string(HomeDir).append("flscope.prefs").c_str(), "w1hkj", PACKAGE_TARNAME, (Fl_Preferences::Root)0);
#endif


	char strbuff[500];
	char version[64];
	version[sizeof(version)-1] = '\0';

	// Skip loading the rest of the status variables if we didn't read a
	// version name/value pair;

	if (!spref.get("version", version, "", sizeof(version)-1))
		return;

	memset(strbuff, 0, sizeof(strbuff));
	spref.get("serial_port", strbuff, "NONE", 499);
	serial_port = strbuff;
	if (serial_port.find("tty") == 0)
		serial_port.insert(0, "/dev/");

	spref.get("main_x", mainX, mainX);
	if (mainX > Fl::w()) mainX = 50;

	spref.get("main_y", mainY, mainY);
	if (mainY > Fl::h()) mainY = 50;

	spref.get("overdrive1", overdrive1, overdrive1);
	spref.get("overdrive2", overdrive2, overdrive2);
	spref.get("alpha", alpha, alpha);
	spref.get("r1qro", r1qro, r1qro);
	spref.get("r1qrp", r1qrp, r1qrp);
	spref.get("targetpwr", targetpwr, targetpwr);
	spref.get("qrp", qrp, qrp);
}
