// ----------------------------------------------------------------------------
// Copyright (C) 2018
//              David Freese, W1HKJ
//
// This file is part of pskmeter.
//
// pskmeter is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// pskmeter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
#include <iostream>
#include <string>

#include "support.h"
#include "main.h"
#include "status.h"
#include "util.h"
#include "comm_support.h"
#include "dataloop.h"

bool debugging = false;

extern void readConfigFile();

Fl_Help_Dialog *help = NULL;
Fl_Window *about = NULL;
Fl_Double_Window *wConfig = 0;

Cserial   *pskserial = 0;
Cpskdata  *pskdata;

bool bViewScope = true;
bool bQRP = false;
bool bAutoDrive = false;
bool bEnabled = true;

double sampletime = 1.0;//0.5;
double saveVol;
double err;

int  iNbrCommPorts  = 0;
int testmode = 0;  // off = 0; idle signal = 1; text signal = 2;

std::string strCommPort;
char szVersion[100];
char progdir[200];

bool connected = false;

bool testMeter()
{
	char buff[1024];
	int n = 0;
	int i = 0;

	memset (buff, 0, 1024);
	for (i = 0; i < 100; i++) {
		pskserial->WriteByte('v');
		MilliSleep(10);
		n = pskserial->ReadBuffer(buff, 1024);
		if (n) break;
	}
	if (!n)
		return false;
	buff[1023] = 0;

	if (strstr(buff,"PSKMETER") != 0)
		return true;
	return false;
}

bool initCommPort()
{
std::cout << "initCommPort() " << strCommPort << std::endl;
	pskserial->Device(strCommPort.c_str());
	pskserial->Baud(9600);
	pskserial->Timeout(100);
	pskserial->Retries(10);
	pskserial->Stopbits(2);//1);
	if (pskserial->OpenPort()) {
		if (testMeter()) {
			btn_connect->value(1);
std::cout << "Connected to nanoIMD on " << strCommPort << std::endl;
			connected = true;
			return true;
		} else {
			pskserial->ClosePort();
			strCommPort.clear();
			progstatus.serial_port.clear();
			fl_message("nanoIMD not on specified port!");
			btn_connect->value(0);
			connected = false;
			return false;
		}
	}
	return false;
}

void initProgram(void *d)
{
	char *executable = (char *)d; 
	strcpy (progdir, executable);
	char *p = strrchr(progdir,'/');
	if (!p) p= strrchr(progdir, '\\');
	if (p) *(p+1) = 0;

	wdgSignalPlot->setData(pskdata);
	wdgSpectrumPlot->setData(pskdata);

	inpCommPort->add(serial_ports().c_str());
	initCommPort();
}

#ifdef __WIN32__
#define exp10(v) exp((v)*log(10))
#endif

void clean_exit()
{
	close_dataloop();

	if (pskserial)
		pskserial->ClosePort();
	progstatus.saveLastState();

	if (help) help->hide();
	if (about) about->hide();
	if (wConfig) wConfig->hide();
	pskscope_dialog->hide();
}

void cb_mnuExit(Fl_Menu_* m, void* d)
{
	clean_exit();
}

void cb_mnuConfig(Fl_Menu_* m, void* d)
{
}

void cb_mnuContext(Fl_Menu_* m, void* d)
{
}

void cb_mnu05sec(Fl_Menu_* m, void* d)
{
	sampletime = 0.5;
}

void cb_mnu10sec(Fl_Menu_* m, void* d)
{
	sampletime = 1.0;
}

void cb_mnu20sec(Fl_Menu_* m, void* d){  
	sampletime = 2.0;
}

void cb_Mode(PSKMODE d)
{
	if (d > PSK63) {
		testmode = d - PSK63;
		pskdata->Mode(PSK31);
	} else {
		testmode = 0;
		pskdata->Mode(d);
	}
}

void cb_mnuConfigGeneral(Fl_Menu_* m, void* d)
{
	valR1qro->value(pskdata->valR1qro());
	valR1qrp->value(pskdata->valR1qrp());
	valSignalMin->value(pskdata->Minval());
	valRed->value(pskdata->Overdrive2());
	valYellow->value(pskdata->Overdrive1());
	valTargetPwr->value(pskdata->targetPwr());
	valFilter->value(pskdata->Alpha());
	inpCommPort->value(progstatus.serial_port.c_str());
	btn_qrp_jumpered->value(pskdata->Qrp());
	wConfig->show();
}

void setParameters()
{
	pskdata->valR1qro(valR1qro->value());
	pskdata->valR1qrp(valR1qrp->value());
	pskdata->Minval(valSignalMin->value());
	pskdata->Overdrive2(valRed->value());
	pskdata->Overdrive1(valYellow->value());
	pskdata->Alpha(valFilter->value());
	pskdata->targetPwr(valTargetPwr->value());
	pskdata->Qrp(btn_qrp_jumpered->value());
}

void writeConfigFile()
{
	progstatus.serial_port = strCommPort;
	progstatus.minval = pskdata->Minval();
	progstatus.overdrive2 = pskdata->Overdrive2();
	progstatus.overdrive1 = pskdata->Overdrive1();
	progstatus.r1qro = pskdata->valR1qro();
	progstatus.r1qrp = pskdata->valR1qrp();
	progstatus.alpha = pskdata->Alpha();
	progstatus.targetpwr =  pskdata->targetPwr();
	progstatus.qrp = pskdata->Qrp();

	progstatus.saveLastState();
}

void readConfigFile()
{
	progstatus.loadLastState();

	strCommPort = progstatus.serial_port;
	if (inpCommPort) {
		inpCommPort->value(progstatus.serial_port.c_str());
	}

	pskdata->Minval(progstatus.minval);
	if (valSignalMin) valSignalMin->value(progstatus.minval);

	pskdata->Overdrive2(progstatus.overdrive2);
	if (valRed) valRed->value(progstatus.overdrive2);

	pskdata->Overdrive1(progstatus.overdrive1);
	if (valYellow) valYellow->value(progstatus.overdrive1);

	pskdata->valR1qro(progstatus.r1qro);
	if (valR1qro) valR1qro->value(progstatus.r1qro);

	pskdata->valR1qrp(progstatus.r1qrp);
	if (valR1qrp) valR1qrp->value(progstatus.r1qrp);

	pskdata->Alpha(progstatus.alpha);
	if (valFilter) valFilter->value(progstatus.alpha);

	pskdata->targetPwr(progstatus.targetpwr);
	if (valTargetPwr) valTargetPwr->value(progstatus.targetpwr);

	pskdata->Qrp(progstatus.qrp);
	if (btn_qrp_jumpered) btn_qrp_jumpered->value(progstatus.qrp);

}

void cb_btnConfigConnect(Fl_Light_Button *b, void *d)
{
	if (pskserial && btn_connect->value() == 0)
		pskserial->ClosePort();
	else {
		strCommPort = inpCommPort->value();
		initCommPort();
	}
}

void cb_btnConfigOK(Fl_Button *b, void * d)
{
	setParameters();
	wConfig->hide();
}

void cb_btnConfigSave(Fl_Button *b, void * d)
{
	writeConfigFile();
}

void cb_wMain(Fl_Double_Window *w, void *d)
{
	if (Fl::event_key() == FL_Escape)
		return;
	clean_exit();
}

//  ============================ About dialog =========================

void cb_mnuAbout(Fl_Menu_ *mnu, void *d)
{
	if (!about) {
		snprintf( szVersion, sizeof(szVersion), "\
flscope %s\n\
Use with Arduino nanoIMD\n\
Free Hamware by W1HKJ\n\n\
Report problems to:", PACKAGE_VERSION);
		about = newAboutDialog ();
		lblVersion->label (szVersion);
	}
	about->show ();
}

void cb_mnuHelpContext (Fl_Menu_ *mnu, void *d)
{
#ifdef __WIN32__
	PROCESS_INFORMATION processInfo;
	STARTUPINFO startupInfo;
	::ZeroMemory(&startupInfo, sizeof(startupInfo));
	startupInfo.cb = sizeof(startupInfo);
	char szCommandLine[160] = "\\Program Files\\Internet Explorer\\IEXPLORE.EXE ";
	strcat(szCommandLine, progdir);
	strcat(szCommandLine, "help\\index.html");
	
	if (!::CreateProcess(
			NULL, szCommandLine,
			NULL, NULL, 
			false, 0, 
			NULL, NULL, 
			&startupInfo, &processInfo))
		fl_message ("Cannot find Internet Explorer");
#else
	fl_message("For help:\n\n"
						 "See  pskscope/help/index.html\n\n");
#endif
	return;
}

void cb_Active(Fl_Check_Button *btn, void *d)
{
	bEnabled = btn->value();
}

void display_data(void *)
{
	char temp[40];
	snprintf(temp, sizeof(temp),"db %s", pskdata->szImd());
	txtIMD->value(temp);
	snprintf(temp, sizeof(temp),"Vp %s", pskdata->szPeak());
	txtPeak->value (temp);
	snprintf(temp, sizeof(temp),"Pw %s",pskdata->szPower());
	txtPower->value (temp);

	wdgSignalPlot->redraw();
	wdgSpectrumPlot->redraw();
	txtRawData->value (pskdata->szRawData());
	txtFilteredData->value (pskdata->szFilteredData());

	pskdata->ready(true);

}
